#!/bin/bash
#
# nfs-chconfig.sh   Do action for firstboot stages.
#
# chkconfig: 2345 80 05
# description: Activates in firstboot stages.
#
#
#===>> do_ima()
#   zhangxu 2017-07-05
#   1. IMA run
#<<=== do_ima() done

#===>> do_selinux()
#   zhangxu 2017-07-05
#   1. just to do selinux work.
#<<=== do_selinux() done

#===>> do_backup_copy_files()
#   zhangxu 2017-07-05
#   1. To move the backup file to backup parts.
#<<=== do_backup_copy_files() done

#===>> do_chage()
#   zhangxu 2017-07-05
#   1. control account password expired.
#<<=== do_chage() done

#===>> do_clean()
#   zhangxu 2017-07-05
#   1. chown -R secadm:secadm /etc/selinux
#   2. do firstboot configure file.
#   3. reboot.
#<<===  do_clean() done

#***************************************************************************************#
# debuginfo wether input /var/log/nfs-touchwork.log
# 1 yes
# 0 no
debuginfo=1

# /var/log/nfs-touchwork.log record nfs-touchwork rpm event and nfs-touchwork.sh (firstboot stage) event infomation.
logfile="/var/log/nfs-touchwork.log"

#***************************************************************************************#

# To move /bin/rpm.ima --> /bin/rpm zhangxu zengyi 2017-07-20 to support install rpm with ima sign.
do_rpmima2rpm()
{
    if [ -f /bin/rpm.ima ];then
        # to copy /bin/rpm.ima --> /bin/rpm
        yes | cp -a /bin/rpm.ima /bin/rpm
        ret=$?
        if [ "$ret" -eq "0" ];then
            echo "success: rpm.ima -->rpm ok"  >> $logfile
        else
            echo "Error: rpm.ima -->rpm faild"  >> $logfile
        fi

        # to del /bin/rpm.ima
        rm -rf /bin/rpm.ima                                                                                                                                   
        ret=$?
        if [ "$ret" -eq "0" ];then
            echo "success: del rpm.ima ok"  >> $logfile
        else
            echo "Error: del rpm.ima faild"  >> $logfile
        fi

        # to del /bin/rpm.orig
        if [ -f /bin/rpm.orig ];then
            rm -rf /bin/rpm.orig
            ret=$?
            if [ "$ret" -eq "0"  ];then
                echo "success:To del /bin/rpm.orig " >> $logfile
            fi
        fi

    else
        echo "Error: rpm.ima is not exist."  >> $logfile
    fi
}

do_ima()
{
    getenforce >> $logfile 2>&1 

    setenforce 0
    systemctl enable smzy_ima.service
    ret=$?
    if [ "$ret" -eq "0" ];then
        if [ ${debuginfo} -eq 1 ];then
            echo -e "DEBUG:[DO_IMA] After setenforce 0 return value $? \n" >> $logfile
            getenforce >> $logfile 2>&1 
        fi   
    fi   

    /usr/bin/setfileima
    ret=$?
    if [ "$ret" -eq "0" ];then
        echo "run ima sign all disk ok." >> $logfile
    fi   
}

do_sm3()
{

        mkdir -p /etc/whitelist/
	file="/evmctl_dir_hash"
        if [ -f $file ]
        then
		sm3sum
        fi
}

do_selinux()
{
    setenforce 0
    ret=$?
    if [ "$ret" -eq "0" ];then
        if [ ${debuginfo} -eq 1 ];then
            echo -e "DEBUG: ready to do selinux setenforce 0 return value $? \n" >> $logfile
            getenforce >> $logfile 2>&1
        fi
    fi

    # genhomedircon
    if [ -e /sbin/restorecon-orig ]
    then
        restorecon-orig -R -F /
    else
        restorecon -R -F /
    fi
    ret=$?
    if [ "$ret" -eq "0" ];then
        echo -e "SUCCESS: DO_SELINUX OK.\n" >> $logfile
    else
        if [ ${debuginfo} -eq 1 ];then
            echo -e "WARNING: Maybe some erro in DO_SELINUX\n" >> $logfile
        fi
    fi
}

do_clean()
{
    # dayong
    chown -R secadm:secadm /etc/selinux
    ret=$?
    if [ "$ret" -ne "0" ];then
        echo -e "Error: chown -R secadm:secadm /etc/selinux failed.\n" >> $logfile
    else
        if [ ${debuginfo} -eq 1 ];then
            echo -e "SUCCESS: chown -R secadm:secadm /etc/selinux. OK\n"  >> $logfile 
        fi
    fi

    # sed -i 's/0077,/0077,rw,context="sysadm_u:object_r:smzy_boot_t:s0",/g' /etc/fstab
}

# To move the backup file to backup parts.
do_backup_copy_files() {

    mount | grep  "recovery" >/dev/null
    ret=$?
    if [ "$ret" -ne "0" ];then
        sd=`mount | grep '/home' | cut -b 6-8`
        sd3=`blkid /dev/${sd}3 | awk -F '"' '{print $2}'`

        mount UUID=$sd3 /recovery
        ret=$?
        if [ ${debuginfo} -eq 1 ];then
            echo -e "mount sda-bkp: return value $ret ....\n" >> $logfile
        fi
    fi 

    
    if [ -d /boot/efi/EFI/NFS/ ]; then 
        # do copy efi mode file for backup.
        sd=`mount | grep '/boot/efi' | cut -b 6-8`
        sd3=`blkid /dev/${sd}3 | awk -F '"' '{print $2}'`
        sed -i 's/dev3UUID/'$sd3'/g' /opt/NFStouchWork/BackupRestore/efi-script/backupgrub.cfg.efi
        yes|mv /opt/NFStouchWork/BackupRestore/efi-script/backupgrub.cfg.efi /boot/efi/EFI/NFS/backupgrub.cfg 
        mv /opt/NFStouchWork/BackupRestore/efi-script/  /recovery/ 
    else
        mv /opt/NFStouchWork/BackupRestore/legacy-script/   /recovery/  
    fi

    mkdir -p /recovery/image/

# JiangChuang 2019-08-21 Added TongFang Zhaoxin Supportion.
    BASEBOARD_PRODUCT_NAME=$(dmidecode -s baseboard-product-name | grep -v '#')
    PROCESSOR_MANUFACTURER=$(dmidecode -s processor-manufacturer | grep -v '#' | uniq)
    SYSTEM_MANUFACTURER=$(dmidecode -s system-manufacturer | grep -v '#')

    if [ "$BASEBOARD_PRODUCT_NAME" = "HX002EF0" -a  "$PROCESSOR_MANUFACTURER" = "Zhaoxin" -a "$SYSTEM_MANUFACTURER" = "THTF" ];then
    	cp -a /opt/NFStouchWork/BackupRestore/nfsbr.iso.tf /recovery/nfsbr.iso
    else
    	cp -a /opt/NFStouchWork/BackupRestore/nfsbr.iso /recovery/
    fi

    cp -a /opt/NFStouchWork/BackupRestore/check_file.sh /recovery/image/

    sync
}


#Step13:To chage user account for control passwd expired
do_chage()
{

    if [ ${debuginfo} -eq 1 ];then
        echo -e "=====================\n" >>  $logfile
    	getenforce >> $logfile 2>&1

    	setenforce 0
	
        date 2>&1 >> $logfile
    fi

    if [ -f /opt/NFStouchWork/CMD/chage  ];then

    	/opt/NFStouchWork/CMD/chage -M 99999 root
    	ret=$?
    	if [ "$ret" -eq "0" ];then
    	    if [ ${debuginfo} -eq 1 ];then
    	        echo -e "SUCCESS: chage -M 30 root\n" >>  $logfile
    	    fi
            # zhangxu 2017-09-06 
            #because chage set root 30 days expired cause root can not login even you input right password.
            #    '012345678912345' 
    	else
    	    echo -e "WARNING: chage -M 30 root \n" >>  $logfile
    	fi

    	/opt/NFStouchWork/CMD/chage -M 30 secadm
    	ret=$?
    	if [ "$ret" -eq "0" ];then
    	    if [ ${debuginfo} -eq 1 ];then
    	        echo -e "SUCCESS: chage -M 30 secadm\n" >>  $logfile
    	    fi
    	else
    	    echo -e "WARNING: chage -M 30 secadm\n" >>  $logfile
    	fi
   
    	/opt/NFStouchWork/CMD/chage -M 30 auditadm
    	ret=$?
    	if [ "$ret" -eq "0" ];then
    	    if [ ${debuginfo} -eq 1 ];then
    	        echo -e "SUCCESS: chage -M 30 auditadm\n" >>  $logfile
    	    fi
    	else
    	    echo -e "WARNING: chage -M 30 auditadm\n" >>  $logfile
    	fi

    fi

    if [ ${debuginfo} -eq 1 ];then
        date 2>&1 >> $logfile
        echo -e "=====================\n" >>  $logfile
    fi

    if [ -f /opt/NFStouchWork/CMD/chage  ];then
        rm -rf /opt/NFStouchWork/CMD/chage
        ret=$?
        if [ "$ret" -eq "0" ];then
            if [ ${debuginfo} -eq 1 ];then
                echo -e "SUCCESS: clean chage ok.\n" >>  $logfile
            fi
        fi
    fi
    
#    if [ -f /opt/NFStouchWork/CMD/chage.nfs ];then
#	chmod u+s /opt/NFStouchWork/CMD/chage.nfs 
#        yes|mv /opt/NFStouchWork/CMD/chage.nfs /usr/bin/chage
#	ret=$?
#        if [ "$ret" -eq "0" ];then
#            if [ ${debuginfo} -eq 1 ];then
#               echo -e "SUCCESS: recover chage ok.\n" >>  $logfile
#            fi
#        fi
#    fi
    if [ -f /usr/bin/chage ]; then
        /usr/bin/chage -M 99999 root
    fi

}

# zhangxu 2018-1-18 we modify useradd init user pwd
# and foabie usermod modify user uid and gid.
do_useradd_usermod()
{
     if [ -f /opt/NFStouchWork/CMD/useradd.initpwd ];then
          yes | mv /opt/NFStouchWork/CMD/useradd.initpwd  /usr/sbin/useradd
          ret=$?
          if [ "$ret" -eq "0" ];then
              if [ ${debuginfo} -eq 1 ];then
                  echo -e "SUCCESS: recover useradd ok.\n" >>  $logfile
              fi
          fi
     fi

     if [ -f /opt/NFStouchWork/CMD/usermod.uidgid ];then
          yes | mv /opt/NFStouchWork/CMD/usermod.uidgid  /usr/sbin/usermod
          ret=$?
          if [ "$ret" -eq "0" ];then
              if [ ${debuginfo} -eq 1 ];then
                  echo -e "SUCCESS: recover usermod ok.\n" >>  $logfile
              fi
          fi
     fi

}

#
# zhangxu 2017-12-20 add supplementary notes
# Prevent do not add pam_card_security.so and pam_time_security.so in /etc/pam.d/system-auth file.
# 
do_system_auth()
{
    if [ -f /etc/pam.d/system-auth ];then
        cat /etc/pam.d/system-auth | grep pam_card_security
        ret=$?
        if [ $ret -ne 0 ];then
            sed -i '4 i \auth     requisite         pam_card_security.so'  /etc/pam.d/system-auth
            sed -i '5 i \auth     requisite         pam_time_security.so'  /etc/pam.d/system-auth
	    sed -i '/^auth.*pam_unix.so.*$/a \auth        required      pam_tally2.so deny=5 unlock_time=1800 even_deny_root root_unlock_time=1800\n\naccount     requisite        pam_tally2.so deny=5 unlock_time=1800  even_deny_root  root_unlock_time=1800'   /etc/pam.d/system-auth

	    sed -i 's/sha512/sha256/g' /etc/pam.d/system-auth
	   
	    sed -i '/pam_sss.so use_authtok/d' /etc/pam.d/system-auth

	    sed -i 's/pam_pwquality.so.*/& retry=2 enforce_for_root/g' /etc/pam.d/system-auth
            echo -e "SUCCESS: system-auth done.\n" >>  $logfile
        fi
    fi
}

#
# gongwen 2020-05-23: replease /etc/pam.d/password-auth with /opt/NFStouchWork/password-auth 
#
do_password_auth()
{
    if [ -f "/opt/NFStouchWork/password-auth" ]; then
	    cp -f /opt/NFStouchWork/password-auth /etc/pam.d/password-auth
    fi

    echo -e "SUCCESS: password-auth done.\n" >>  $logfile

}

#
# gongwen 2020-05-21: add pam_card_security.so and pam_time_security.so in /etc/pam.d/gdm-password
#
do_gdm_password()
{
    if [ -f /etc/pam.d/gdm-password ];then
        cat /etc/pam.d/gdm-password | grep pam_card_security
        ret=$?

	if [ $ret -ne 0 ];then
	    sed -i '1 i \auth     required         pam_card_security.so' /etc/pam.d/gdm-password
	    sed -i '2 i \auth     required         pam_time_security.so' /etc/pam.d/gdm-password

            echo -e "SUCCESS: gdm-password done.\n" >>  $logfile

	fi
    fi
}

#
# gongwen 2020-05-21: add pam_card_security.so and pam_time_security.so in /etc/pam.d/sshd
#
do_sshd()
{
    if [ -f /etc/pam.d/sshd ];then
        cat /etc/pam.d/sshd | grep pam_card_security
        ret=$?

	if [ $ret -ne 0 ];then
	    sed -i '2 i \auth     required         pam_card_security.so' /etc/pam.d/sshd

	fi

	cat /etc/pam.d/sshd | grep pam_time_security
	ret2=$?

	if [ $ret2 -ne 0 ];then
	    sed -i '3 i \auth     required         pam_time_security.so' /etc/pam.d/sshd
	fi

        echo -e "SUCCESS: sshd done.\n" >>  $logfile

    fi
}

do_su()
{
    if [ -f /etc/pam.d/su ];then
	sed -i '$a \session     required	pam_selinux.so open' /etc/pam.d/su 
	sed -i '$a \session     required	pam_selinux.so close' /etc/pam.d/su

        echo -e "SUCCESS: su done.\n" >>  $logfile
    fi

}

do_cockpit()
{
    if [ -f /etc/pam.d/cockpit ];then
        sed -i '2 i \auth     requisite         pam_card_security.so'  /etc/pam.d/cockpit
        sed -i '3 i \auth     requisite         pam_time_security.so'  /etc/pam.d/cockpit
        sed -i '4 i \auth        required      pam_tally2.so deny=5 unlock_time=1800 even_deny_root root_unlock_time=1800'   /etc/pam.d/cockpit
	
        echo -e "SUCCESS: su done.\n" >>  $logfile

    fi

}

do_remove_debugfs()
{
    if [ -f /usr/sbin/debugfs ];then
	rm -f /usr/sbin/debugfs
        echo -e "SUCCESS: do_remove_debugfs done.\n" >>  $logfile

    fi
}

do_remove_semanage()
{
    if [ -f /usr/sbin/semanage ];then
	rm -f /usr/sbin/semanage
        echo -e "SUCCESS: do_remove_semanage done.\n" >>  $logfile

    fi

    if [ -f /usr/bin/chcat ];then
	rm -f /usr/bin/chcat
        echo -e "SUCCESS: do_remove_chcat done.\n" >>  $logfile

    fi
}

do_remove_restorcond_desktop()
{
    if [ -f /etc/xdg/autostart/restorecond.desktop ];then
	rm -f /etc/xdg/autostart/restorecond.desktop
        echo -e "SUCCESS: do_remove_restorcond_desktop done.\n" >>  $logfile
	
    fi

}

do_add_link_for_vmlinuz()
{
    link_for_vmlinuz=`ls /boot | grep vmlinuz | grep -v rescue | grep -v kdump`

    if [ -n "$link_for_vmlinuz" ];then
    	ln -s /boot/${link_for_vmlinuz} /boot/vmlinuz-3.10.0-514.26.2.nfs.25.x86_64

    fi

    link_for_initramfs=`ls /boot | grep initramfs | grep -v rescue | grep -v kdump`
   
    if [ -n "$link_for_initramfs" ];then
    	ln -s /boot/${link_for_initramfs} /boot/initramfs-3.10.0-514.26.2.nfs.25.x86
_64.img
    fi

    echo -e "SUCCESS: do_add_link_for_vmlinuz done.\n" >>  $logfile

}

do_fix_httcgui()
{

    if [ -f /root/桌面/httcgui.desktop ];then
	chmod a+x /root/桌面/httcgui.desktop

    	echo -e "SUCCESS: do_fix_httcgui done.\n" >>  $logfile
    fi 


}

do_system_services_link()
{
	systemd_dir="/usr/lib/systemd/system"
	for f in `find /etc/systemd/system/ -type f`
	do
		dir=`dirname $f`
		name=`basename $f`
		if [ `basename $dir` = "system" ]
		then
			continue
		fi
		
		rm -rf $systemd_dir/$name
		mv $f $systemd_dir/$name
		ln -s $systemd_dir/$name $f
	done

	systemctl disable radvd.service
	systemctl disable postfix.service
	systemctl disable dnsmasq.service
	systemctl disable postgresql.service
	systemctl disable firewalld.service
	systemctl disable gssproxy.service
	systemctl disable spamassassin.service
	systemctl disable iscsid.service
	systemctl disable iscsiuio.service
	systemctl disable tomcat.service
	systemctl disable keepalived.service
	systemctl disable mariadb.service
	systemctl disable named.service
	systemctl disable named-chroot.service
	systemctl disable named-chroot-setup.service
	systemctl disable named-setup-rndc.service
	systemctl disable nfs-server.service
	systemctl disable custodia@.service
	systemctl disable custodia@.socket
	systemctl disable pmlogger.service

}

case "$1" in
    start)
        # to do
        # add judge the secure server only run once.
	#13 To move the backup file to backup parts.

        #hide the applications,such as:policyeditor  itweb-settings openjdk 
	rm -rf /usr/share/applications/policyeditor.desktop
	rm -rf /usr/share/applications/itweb-settings.desktop
	rm -rf /usr/share/applications/java-1.8.0-openjdk-1.8.0.242.b08-0.nfs.x86_64-jconsole.desktop
	rm -rf /usr/share/applications/java-1.8.0-openjdk-1.8.0.242.b08-0.nfs.x86_64-policytool.desktop

	do_system_services_link

	# echo -e "\n\n[do_chage]\n" >> $logfile
    do_chage

	#6 remove restorcond.desktop
	echo -e "\n\n[do_remove_restorcond_desktop]\n" >> $logfile
	do_remove_restorcond_desktop

	#7 add link for vmlinuz & initramfs
	do_add_link_for_vmlinuz

        #8 IMA run.
        echo -e "\n\n[do_ima]\n" >> $logfile
        do_ima

        #10 Selinux.
        echo -e "\n\n[do_selinux]\n" >> $logfile
        do_selinux

	#13 To move the backup file to backup parts.
	echo -e "\n\n[do_backup_copy_files]\n" >> $logfile
	do_backup_copy_files

        #15 Step-Last: To clean
        echo -e "\n\n[do_clean]\n" >> $logfile
        do_clean

    ;;
    stop)
        echo "This script run only once"
    ;;
    *)
        echo "Usage: $0 start"
        exit 1
    ;;
esac


